@@ -44,57 +44,16 @@ static bool _wlmaker_config_decode_fill_style(
 
 /* == Data ================================================================= */
 
-/** Name of the xcursor theme. NULL picks the default. */
-const char *config_xcursor_theme_name = NULL;
-
-/** Base size for the xcursor theme (when scale==1.0). */
-const uint32_t config_xcursor_theme_size = 24;
-
 /** Overall scale of output. */
 const float config_output_scale = 1.0;
 
-/** Whether to always request server-side decorations. */
-const wlmaker_config_decoration_t config_decoration =
-    WLMAKER_CONFIG_DECORATION_SUGGEST_SERVER;
-
-/** Modifiers for moving the window with the cursor. */
-const uint32_t wlmaker_config_window_drag_modifiers =
-    WLR_MODIFIER_ALT | WLR_MODIFIER_LOGO;
-
-/** Visual theme. */
-const wlmaker_config_theme_t  wlmaker_config_theme = {
-    .window_margin_color = 0xff000000,  // Pich black, opaque.
-    .window_margin_width = 1,
-
-    .tile_fill = {
-        .type = WLMTK_STYLE_COLOR_DGRADIENT,
-        .param = { .hgradient = { .from = 0xffa6a6b6,.to = 0xff515561 }}
-    },
-    .menu_fill = {
-        .type = WLMTK_STYLE_COLOR_HGRADIENT,
-        .param = { .hgradient = { .from = 0xffc2c0c5, .to = 0xff828085 }}
-    },
-    .menu_margin_color = 0xff000000,  // Pitch black, opaque.
-    .menu_margin_width = 1,
-    .menu_padding_width = 1,
-
-    .menu_item_enabled_fill = {
-        .type = WLMTK_STYLE_COLOR_SOLID,
-        .param = { .solid = { .color = 0x00000000 }}  // Transparent.
-    },
-    .menu_item_enabled_text_color = 0xff000000,  // Black, opaque.
-    .menu_item_selected_fill = {
-        .type = WLMTK_STYLE_COLOR_SOLID,
-        .param = { .solid = { .color = 0xffffffff }}  // White, opaque..
-    },
-    .menu_item_selected_text_color = 0xff000000,  // Black, opaque.
-};
-
 /** Plist decoding descriptor of the fill type. */
 static const wlmcfg_enum_desc_t _wlmaker_config_fill_type_desc[] = {
     WLMCFG_ENUM("SOLID", WLMTK_STYLE_COLOR_SOLID),
     WLMCFG_ENUM("HGRADIENT", WLMTK_STYLE_COLOR_HGRADIENT),
+    WLMCFG_ENUM("VGRADIENT", WLMTK_STYLE_COLOR_VGRADIENT),
     WLMCFG_ENUM("DGRADIENT", WLMTK_STYLE_COLOR_DGRADIENT),
+    WLMCFG_ENUM("ADGRADIENT", WLMTK_STYLE_COLOR_ADGRADIENT),
     WLMCFG_ENUM_SENTINEL()
 };
 
@@ -133,6 +92,8 @@ static const wlmcfg_desc_t _wlmaker_config_style_color_gradient_desc[] = {
 static const wlmcfg_desc_t _wlmaker_config_tile_style_desc[] = {
     WLMCFG_DESC_UINT64(
         "Size", true, wlmtk_tile_style_t, size, 64),
+    WLMCFG_DESC_UINT64(
+        "ContentSize", true, wlmtk_tile_style_t, content_size, 48),
     WLMCFG_DESC_UINT64(
         "BezelWidth", true, wlmtk_tile_style_t, bezel_width, 2),
     WLMCFG_DESC_CUSTOM(
@@ -210,7 +171,7 @@ static const wlmcfg_desc_t _wlmaker_config_window_resize_style_desc[] = {
     WLMCFG_DESC_UINT64(
         "CornerWidth", true, wlmtk_resizebar_style_t, corner_width, 1),
     WLMCFG_DESC_SENTINEL()
- };
+};
 
 /** Descriptor for decoding the "Window" dictionary. */
 static const wlmcfg_desc_t _wlmaker_config_window_style_desc[] = {
@@ -229,6 +190,49 @@ static const wlmcfg_desc_t _wlmaker_config_window_style_desc[] = {
     WLMCFG_DESC_SENTINEL()
 };
 
+/** Descriptor for decoding the "Item" dictionary. */
+static const wlmcfg_desc_t _wlmaker_config_menu_item_style_desc[] = {
+    WLMCFG_DESC_CUSTOM(
+        "Fill", true, wlmtk_menu_item_style_t, fill,
+        _wlmaker_config_decode_fill_style, NULL, NULL),
+    WLMCFG_DESC_CUSTOM(
+        "HighlightedFill", true, wlmtk_menu_item_style_t, highlighted_fill,
+        _wlmaker_config_decode_fill_style, NULL, NULL),
+    WLMCFG_DESC_DICT(
+        "Font", true, wlmtk_menu_item_style_t, font,
+        _wlmaker_config_font_style_desc),
+    WLMCFG_DESC_ARGB32(
+        "EnabledTextColor", true, wlmtk_menu_item_style_t,
+        enabled_text_color, 0),
+    WLMCFG_DESC_ARGB32(
+        "HighlightedTextColor", true, wlmtk_menu_item_style_t,
+        highlighted_text_color, 0),
+    WLMCFG_DESC_ARGB32(
+        "DisabledTextColor", true, wlmtk_menu_item_style_t,
+        disabled_text_color, 0),
+    WLMCFG_DESC_UINT64(
+        "Height", true, wlmtk_menu_item_style_t, height, 20),
+    WLMCFG_DESC_UINT64(
+        "BezelWidth", true, wlmtk_menu_item_style_t, bezel_width, 1),
+    WLMCFG_DESC_UINT64(
+        "Width", true, wlmtk_menu_item_style_t, width, 80),
+    WLMCFG_DESC_SENTINEL()
+};
+
+/** Descriptor for decoding the "Menu" dictionary. */
+static const wlmcfg_desc_t _wlmaker_config_menu_style_desc[] = {
+    WLMCFG_DESC_DICT(
+        "Item", true, wlmtk_menu_style_t, item,
+        _wlmaker_config_menu_item_style_desc),
+    WLMCFG_DESC_DICT(
+        "Margin", true, wlmtk_menu_style_t, margin,
+        _wlmaker_config_margin_style_desc),
+    WLMCFG_DESC_DICT(
+        "Border", true, wlmtk_menu_style_t, border,
+        _wlmaker_config_margin_style_desc),
+    WLMCFG_DESC_SENTINEL()
+};
+
 /** Descriptor for decoding the "TaskList" dictionary. */
 static const wlmcfg_desc_t _wlmaker_task_list_style_desc[] = {
     WLMCFG_DESC_CUSTOM(
@@ -254,6 +258,15 @@ static const wlmcfg_desc_t _wlmaker_clip_style_desc[] = {
     WLMCFG_DESC_SENTINEL()
 };
 
+/** Descriptor for decoding the "Cursor" dictionary. */
+static const wlmcfg_desc_t _wlmaker_cursor_style_desc[] = {
+    WLMCFG_DESC_STRING(
+        "Name", true, wlmaker_config_cursor_style_t, name_ptr, "default"),
+    WLMCFG_DESC_UINT64(
+        "Size", true, wlmaker_config_cursor_style_t, size, 24),
+    WLMCFG_DESC_SENTINEL()
+};
+
 /** Desciptor for decoding the style information from a plist. */
 const wlmcfg_desc_t wlmaker_config_style_desc[] = {
     WLMCFG_DESC_ARGB32(
@@ -267,12 +280,18 @@ const wlmcfg_desc_t wlmaker_config_style_desc[] = {
     WLMCFG_DESC_DICT(
         "Window", true, wlmaker_config_style_t, window,
         _wlmaker_config_window_style_desc),
+    WLMCFG_DESC_DICT(
+        "Menu", true, wlmaker_config_style_t, menu,
+        _wlmaker_config_menu_style_desc),
     WLMCFG_DESC_DICT(
         "TaskList", true, wlmaker_config_style_t, task_list,
         _wlmaker_task_list_style_desc),
     WLMCFG_DESC_DICT(
         "Clip", true, wlmaker_config_style_t, clip,
         _wlmaker_clip_style_desc),
+    WLMCFG_DESC_DICT(
+        "Cursor", true, wlmaker_config_style_t, cursor,
+        _wlmaker_cursor_style_desc),
     WLMCFG_DESC_SENTINEL()
 };
 
@@ -312,8 +331,8 @@ wlmcfg_dict_t *wlmaker_config_load(const char *fname_ptr)
         // If we get here, there was a resolved item at the path. A load
         // failure indicates an issue with an existing file, and we should
         // fali here.
-        bs_log(BS_INFO, "Loading configuration from \"%s\"", *fname_ptr_ptr);
-        return _wlmaker_config_from_plist(*fname_ptr_ptr);
+        bs_log(BS_INFO, "Loading configuration from \"%s\"", path_ptr);
+        return _wlmaker_config_from_plist(path_ptr);
     }
 
     // Hardcoded configuration. Failing to load that is an error.
@@ -346,8 +365,8 @@ wlmcfg_dict_t *wlmaker_state_load(const char *fname_ptr)
         // If we get here, there was a resolved item at the path. A load
         // failure indicates an issue with an existing file, and we should
         // fali here.
-        bs_log(BS_INFO, "Loading state from \"%s\"", *fname_ptr_ptr);
-        return _wlmaker_config_from_plist(*fname_ptr_ptr);
+        bs_log(BS_INFO, "Loading state from \"%s\"", path_ptr);
+        return _wlmaker_config_from_plist(path_ptr);
     }
 
     // Hardcoded configuration. Failing to load that is an error.
@@ -409,16 +428,26 @@ bool _wlmaker_config_decode_fill_style(
             dict_ptr,
             _wlmaker_config_style_color_solid_desc,
             &fill_ptr->param.solid);
-    case WLMTK_STYLE_COLOR_DGRADIENT:
-        return wlmcfg_decode_dict(
-            dict_ptr,
-            _wlmaker_config_style_color_gradient_desc,
-            &fill_ptr->param.dgradient);
     case WLMTK_STYLE_COLOR_HGRADIENT:
         return wlmcfg_decode_dict(
             dict_ptr,
             _wlmaker_config_style_color_gradient_desc,
             &fill_ptr->param.hgradient);
+    case WLMTK_STYLE_COLOR_VGRADIENT:
+        return wlmcfg_decode_dict(
+            dict_ptr,
+            _wlmaker_config_style_color_gradient_desc,
+            &fill_ptr->param.vgradient);
+    case WLMTK_STYLE_COLOR_DGRADIENT:
+        return wlmcfg_decode_dict(
+            dict_ptr,
+            _wlmaker_config_style_color_gradient_desc,
+            &fill_ptr->param.dgradient);
+    case WLMTK_STYLE_COLOR_ADGRADIENT:
+        return wlmcfg_decode_dict(
+            dict_ptr,
+            _wlmaker_config_style_color_gradient_desc,
+            &fill_ptr->param.adgradient);
     default:
         bs_log(BS_ERROR, "Unhandled fill type %d", fill_ptr->type);
         return false;
@@ -505,7 +534,7 @@ void test_file(bs_test_t *test_ptr)
 void test_style_file(bs_test_t *test_ptr)
 {
     wlmcfg_dict_t *dict_ptr;
-    wlmaker_config_style_t config_style;
+    wlmaker_config_style_t config_style = {};
 
 #ifndef WLMAKER_SOURCE_DIR
 #error "Missing definition of WLMAKER_SOURCE_DIR!"
