@@ -24,6 +24,7 @@
 #include "gfxbuf.h"
 #include "util.h"
 
+#include <wlr/version.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_scene.h>
@@ -79,6 +80,12 @@ static void _wlmtk_surface_handle_wlr_scene_tree_node_destroy(
 static void _wlmtk_surface_handle_surface_commit(
     struct wl_listener *listener_ptr,
     void *data_ptr);
+static void _wlmtk_surface_handle_surface_map(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+static void _wlmtk_surface_handle_surface_unmap(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
 
 static void _wlmtk_surface_commit_size(
     wlmtk_surface_t *surface_ptr,
@@ -158,17 +165,19 @@ void wlmtk_surface_set_activated(
                 wlr_keyboard_ptr->keycodes,
                 wlr_keyboard_ptr->num_keycodes,
                 &wlr_keyboard_ptr->modifiers);
+
+        }
+        if (NULL != surface_ptr->super_element.parent_container_ptr) {
+            wlmtk_container_set_keyboard_focus_element(
+                surface_ptr->super_element.parent_container_ptr,
+                &surface_ptr->super_element);
         }
-        wlmtk_container_update_keyboard_focus(
-            surface_ptr->super_element.parent_container_ptr,
-            &surface_ptr->super_element);
     } else {
         if (wlr_seat_ptr->keyboard_state.focused_surface ==
             surface_ptr->wlr_surface_ptr) {
             wlr_seat_keyboard_clear_focus(wlr_seat_ptr);
-            wlmtk_container_update_keyboard_focus(
-                surface_ptr->super_element.parent_container_ptr,
-                NULL);
+            wlmtk_container_set_keyboard_focus_element(
+                surface_ptr->super_element.parent_container_ptr, NULL);
         }
     }
 
@@ -235,6 +244,14 @@ bool _wlmtk_surface_init(
             &wlr_surface_ptr->events.commit,
             &surface_ptr->surface_commit_listener,
             _wlmtk_surface_handle_surface_commit);
+        wlmtk_util_connect_listener_signal(
+            &wlr_surface_ptr->events.map,
+            &surface_ptr->surface_map_listener,
+            _wlmtk_surface_handle_surface_map);
+        wlmtk_util_connect_listener_signal(
+            &wlr_surface_ptr->events.unmap,
+            &surface_ptr->surface_unmap_listener,
+            _wlmtk_surface_handle_surface_unmap);
     }
     return true;
 }
@@ -255,6 +272,8 @@ void _wlmtk_surface_fini(wlmtk_surface_t *surface_ptr)
     if (NULL != surface_ptr->wlr_surface_ptr) {
         surface_ptr->wlr_surface_ptr = NULL;
         wlmtk_util_disconnect_listener(&surface_ptr->surface_commit_listener);
+        wlmtk_util_disconnect_listener(&surface_ptr->surface_map_listener);
+        wlmtk_util_disconnect_listener(&surface_ptr->surface_unmap_listener);
     }
 
     wlmtk_element_fini(&surface_ptr->super_element);
@@ -493,12 +512,20 @@ bool _wlmtk_surface_element_pointer_button(
     // We're only forwarding PRESSED & RELEASED events.
     if (WLMTK_BUTTON_DOWN == button_event_ptr->type ||
         WLMTK_BUTTON_UP == button_event_ptr->type) {
+#if WLR_VERSION_NUM >= (18 << 8)
+        enum wl_pointer_button_state state =
+            (button_event_ptr->type == WLMTK_BUTTON_DOWN) ?
+            WL_POINTER_BUTTON_STATE_PRESSED : WL_POINTER_BUTTON_STATE_RELEASED;
+#else // WLR_VERSION_NUM >= (18 << 8)
+        enum wlr_button_state state =
+            (button_event_ptr->type == WLMTK_BUTTON_DOWN) ?
+            WLR_BUTTON_PRESSED : WLR_BUTTON_RELEASED;
+#endif // WLR_VERSION_NUM >= (18 << 8)
         wlr_seat_pointer_notify_button(
             wlmtk_env_wlr_seat(surface_ptr->super_element.env_ptr),
             button_event_ptr->time_msec,
             button_event_ptr->button,
-            (button_event_ptr->type == WLMTK_BUTTON_DOWN) ?
-            WLR_BUTTON_PRESSED : WLR_BUTTON_RELEASED);
+            state);
         return true;
     }
     return false;
@@ -535,7 +562,11 @@ bool _wlmtk_surface_element_pointer_axis(
         wlr_pointer_axis_event_ptr->orientation,
         wlr_pointer_axis_event_ptr->delta,
         wlr_pointer_axis_event_ptr->delta_discrete,
-        wlr_pointer_axis_event_ptr->source);
+        wlr_pointer_axis_event_ptr->source
+#if WLR_VERSION_NUM >= (18 << 8)
+        , wlr_pointer_axis_event_ptr->relative_direction
+#endif // WLR_VERSION_NUM >= (18 << 8)
+        );
     return true;
 }
 
@@ -620,6 +651,39 @@ void _wlmtk_surface_handle_surface_commit(
         surface_ptr->wlr_surface_ptr->current.height);
 }
 
+/* ------------------------------------------------------------------------- */
+/**
+ * Handles the `surface_map` signal: Makes the surface visible.
+ *
+ * @param listener_ptr
+ * @param data_ptr
+ */
+void _wlmtk_surface_handle_surface_map(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
+{
+    wlmtk_surface_t *surface_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmtk_surface_t, surface_map_listener);
+    wlmtk_element_set_visible(wlmtk_surface_element(surface_ptr), true);
+}
+
+/* ------------------------------------------------------------------------- */
+/**
+ * Handles the `surface_unmap` signal: Makes the surface invisible.
+ *
+ * @param listener_ptr
+ * @param data_ptr
+ */
+void _wlmtk_surface_handle_surface_unmap(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
+{
+    wlmtk_surface_t *surface_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmtk_surface_t, surface_unmap_listener);
+    wlmtk_element_set_visible( wlmtk_surface_element(surface_ptr), false);
+}
+
+
 /* ------------------------------------------------------------------------- */
 /**
  * Surface commits a new size: Store the size, and update the parent's layout.
