@@ -51,6 +51,8 @@ struct _wlmtk_workspace_t {
     /** Current FSM state. */
     wlmtk_fsm_t               fsm;
 
+    /** Whether this workspace is enabled, ie. can have activated windows. */
+    bool                      enabled;
     /** Container that holds the windows, ie. the window layer. */
     wlmtk_container_t         window_container;
     /** Container that holds the fullscreen elements. Should have only one. */
@@ -61,6 +63,8 @@ struct _wlmtk_workspace_t {
 
     /** The activated window. */
     wlmtk_window_t            *activated_window_ptr;
+    /** The most recent activated window, if none is activated now. */
+    wlmtk_window_t            *formerly_activated_window_ptr;
 
     /** The grabbed window. */
     wlmtk_window_t            *grabbed_window_ptr;
@@ -96,6 +100,9 @@ struct _wlmtk_workspace_t {
     wlmtk_layer_t             *top_layer_ptr;
     /** Overlay layer. */
     wlmtk_layer_t             *overlay_layer_ptr;
+
+    /** Copy of the tile's style, for dimensions; */
+    wlmtk_tile_style_t        tile_style;
 };
 
 static void _wlmtk_workspace_element_destroy(wlmtk_element_t *element_ptr);
@@ -173,11 +180,13 @@ static const wlmtk_fsm_transition_t pfsm_transitions[] = {
 /* ------------------------------------------------------------------------- */
 wlmtk_workspace_t *wlmtk_workspace_create(
     const char *name_ptr,
+    const wlmtk_tile_style_t *tile_style_ptr,
     wlmtk_env_t *env_ptr)
 {
     wlmtk_workspace_t *workspace_ptr =
         logged_calloc(1, sizeof(wlmtk_workspace_t));
     if (NULL == workspace_ptr) return NULL;
+    workspace_ptr->tile_style = *tile_style_ptr;
     workspace_ptr->name_ptr = logged_strdup(name_ptr);
     if (NULL == workspace_ptr->name_ptr) {
         wlmtk_workspace_destroy(workspace_ptr);
@@ -385,8 +394,10 @@ struct wlr_box wlmtk_workspace_get_maximize_extents(
     struct wlr_box box = {
         .x = workspace_ptr->x1,
         .y = workspace_ptr->y1,
-        .width = workspace_ptr->x2 - workspace_ptr->x1 - 64,
-        .height = workspace_ptr->y2 - workspace_ptr->y1 - 64 };
+        .width = (workspace_ptr->x2 - workspace_ptr->x1 -
+                  workspace_ptr->tile_style.size),
+        .height = (workspace_ptr->y2 - workspace_ptr->y1 -
+                   workspace_ptr->tile_style.size) };
     return box;
 }
 
@@ -402,6 +413,21 @@ struct wlr_box wlmtk_workspace_get_fullscreen_extents(
     return box;
 }
 
+/* ------------------------------------------------------------------------- */
+void wlmtk_workspace_enable(wlmtk_workspace_t *workspace_ptr, bool enabled)
+{
+    if (workspace_ptr->enabled == enabled) return;
+
+    workspace_ptr->enabled = enabled;
+    if (!enabled) {
+        wlmtk_workspace_activate_window(workspace_ptr, NULL);
+    } else {
+        wlmtk_workspace_activate_window(
+            workspace_ptr,
+            workspace_ptr->formerly_activated_window_ptr);
+    }
+}
+
 /* ------------------------------------------------------------------------- */
 void wlmtk_workspace_map_window(wlmtk_workspace_t *workspace_ptr,
                                 wlmtk_window_t *window_ptr)
@@ -442,6 +468,10 @@ void wlmtk_workspace_unmap_window(wlmtk_workspace_t *workspace_ptr,
         wlmtk_workspace_activate_window(workspace_ptr, NULL);
         need_activation = true;
     }
+    if (workspace_ptr->formerly_activated_window_ptr == window_ptr) {
+        workspace_ptr->formerly_activated_window_ptr = NULL;
+        need_activation = true;
+    }
 
     wlmtk_element_set_visible(wlmtk_window_element(window_ptr), false);
 
@@ -586,12 +616,17 @@ void wlmtk_workspace_activate_window(
 
     if (NULL != workspace_ptr->activated_window_ptr) {
         wlmtk_window_set_activated(workspace_ptr->activated_window_ptr, false);
+        workspace_ptr->formerly_activated_window_ptr =
+            workspace_ptr->activated_window_ptr;
         workspace_ptr->activated_window_ptr = NULL;
     }
 
     if (NULL != window_ptr) {
-        wlmtk_window_set_activated(window_ptr, true);
-        workspace_ptr->activated_window_ptr = window_ptr;
+        if (workspace_ptr->enabled) {
+            wlmtk_window_set_activated(window_ptr, true);
+            workspace_ptr->activated_window_ptr = window_ptr;
+        }
+        workspace_ptr->formerly_activated_window_ptr = window_ptr;
     }
 }
 
@@ -691,55 +726,22 @@ wlmtk_workspace_t *wlmtk_workspace_from_dlnode(
     return BS_CONTAINER_OF(dlnode_ptr, wlmtk_workspace_t, dlnode);
 }
 
-/* == Fake workspace methods, useful for tests ============================= */
-
 /* ------------------------------------------------------------------------- */
-wlmtk_fake_workspace_t *wlmtk_fake_workspace_create(int width, int height)
+wlmtk_workspace_t *wlmtk_workspace_create_for_test(
+    int width,
+    int height,
+    wlmtk_env_t *env_ptr)
 {
-    wlmtk_fake_workspace_t *fw_ptr = logged_calloc(
-        1, sizeof(wlmtk_fake_workspace_t));
-    if (NULL == fw_ptr) return NULL;
-
-    fw_ptr->workspace_ptr = wlmtk_workspace_create("fake", NULL);
-    if (NULL == fw_ptr->workspace_ptr) {
-        wlmtk_fake_workspace_destroy(fw_ptr);
-        return NULL;
-    }
-    wlmtk_element_set_visible(
-        wlmtk_workspace_element(fw_ptr->workspace_ptr), true);
-
-    fw_ptr->fake_parent_ptr = wlmtk_container_create_fake_parent();
-    if (NULL == fw_ptr->fake_parent_ptr) {
-        wlmtk_fake_workspace_destroy(fw_ptr);
-        return NULL;
-    }
-    wlmtk_container_add_element(
-        fw_ptr->fake_parent_ptr,
-        wlmtk_workspace_element(fw_ptr->workspace_ptr));
+    static const wlmtk_tile_style_t ts = { .size = 64 };
+    wlmtk_workspace_t *workspace_ptr = wlmtk_workspace_create(
+        "test", &ts, env_ptr);
+    if (NULL == workspace_ptr) return NULL;
 
     struct wlr_box extents = { .width = width, .height = height };
-    wlmtk_workspace_set_extents(fw_ptr->workspace_ptr, &extents);
+    wlmtk_workspace_set_extents(workspace_ptr, &extents);
+    wlmtk_workspace_enable(workspace_ptr, true);
 
-    return fw_ptr;
-}
-
-/* ------------------------------------------------------------------------- */
-void wlmtk_fake_workspace_destroy(wlmtk_fake_workspace_t *fw_ptr)
-{
-    if (NULL != fw_ptr->fake_parent_ptr) {
-        wlmtk_container_remove_element(
-            fw_ptr->fake_parent_ptr,
-            wlmtk_workspace_element(fw_ptr->workspace_ptr));
-        wlmtk_container_destroy_fake_parent(fw_ptr->fake_parent_ptr);
-        fw_ptr->fake_parent_ptr = NULL;
-    }
-
-    if (NULL != fw_ptr->workspace_ptr) {
-        wlmtk_workspace_destroy(fw_ptr->workspace_ptr);
-        fw_ptr->workspace_ptr = NULL;
-    }
-
-    free(fw_ptr);
+    return workspace_ptr;
 }
 
 /* == Local (static) methods =============================================== */
@@ -990,6 +992,7 @@ static void test_map_unmap(bs_test_t *test_ptr);
 static void test_move(bs_test_t *test_ptr);
 static void test_unmap_during_move(bs_test_t *test_ptr);
 static void test_resize(bs_test_t *test_ptr);
+static void test_enable(bs_test_t *test_ptr);
 static void test_activate(bs_test_t *test_ptr);
 static void test_activate_cycling(bs_test_t *test_ptr);
 
@@ -999,6 +1002,7 @@ const bs_test_case_t wlmtk_workspace_test_cases[] = {
     { 1, "move", test_move },
     { 1, "unmap_during_move", test_unmap_during_move },
     { 1, "resize", test_resize },
+    { 1, "enable", test_enable } ,
     { 1, "activate", test_activate },
     { 1, "activate_cycling", test_activate_cycling },
     { 0, NULL, NULL }
@@ -1040,14 +1044,17 @@ static void _wlmtk_workspace_test_handle_window_unmapped(
     tl_ptr->window_unmapped_handler_invoked = true;
 }
 
+/** Tile style used in tests. */
+static const wlmtk_tile_style_t _wlmtk_workspace_test_tile_style = {
+    .size = 64
+};
+
 /* ------------------------------------------------------------------------- */
 /** Exercises workspace create & destroy methods. */
 void test_create_destroy(bs_test_t *test_ptr)
 {
-    wlmtk_container_t *fake_parent_ptr = wlmtk_container_create_fake_parent();
-    BS_ASSERT(NULL != fake_parent_ptr);
-
-    wlmtk_workspace_t *workspace_ptr = wlmtk_workspace_create("test", NULL);
+    wlmtk_workspace_t *workspace_ptr = wlmtk_workspace_create(
+        "test", &_wlmtk_workspace_test_tile_style, NULL);
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, workspace_ptr);
 
     struct wlr_box box = { .x = -10, .y = -20, .width = 100, .height = 200 };
@@ -1080,7 +1087,6 @@ void test_create_destroy(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(test_ptr, 42, index);
 
     wlmtk_workspace_destroy(workspace_ptr);
-    wlmtk_container_destroy_fake_parent(fake_parent_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -1091,7 +1097,8 @@ void test_map_unmap(bs_test_t *test_ptr)
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wlr_scene_ptr);
     wlmtk_root_t *root_ptr = wlmtk_root_create(wlr_scene_ptr, NULL);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, root_ptr);
-    wlmtk_workspace_t *workspace_ptr = wlmtk_workspace_create("test", NULL);
+    wlmtk_workspace_t *workspace_ptr = wlmtk_workspace_create(
+        "test", &_wlmtk_workspace_test_tile_style, NULL);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, workspace_ptr);
     wlmtk_root_add_workspace(root_ptr, workspace_ptr);
 
@@ -1109,7 +1116,7 @@ void test_map_unmap(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(test_ptr, 0, bs_dllist_size(wdl_ptr));
 
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
 
     BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_window_element(fw_ptr->window_ptr)->visible);
     wlmtk_workspace_map_window(workspace_ptr, fw_ptr->window_ptr);
@@ -1146,29 +1153,29 @@ void test_map_unmap(bs_test_t *test_ptr)
     wlmtk_root_remove_workspace(root_ptr, workspace_ptr);
     wlmtk_workspace_destroy(workspace_ptr);
     wlmtk_root_destroy(root_ptr);
-    free(wlr_scene_ptr);
+    wlr_scene_node_destroy(&wlr_scene_ptr->tree.node);
 }
 
 /* ------------------------------------------------------------------------- */
 /** Tests moving a window. */
 void test_move(bs_test_t *test_ptr)
 {
-    wlmtk_fake_workspace_t *fws_ptr = wlmtk_fake_workspace_create(1024, 768);
-    BS_ASSERT(NULL != fws_ptr);
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
 
     wlmtk_element_pointer_motion(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr), 0, 0, 42);
+        wlmtk_workspace_element(ws_ptr), 0, 0, 42);
 
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw_ptr->window_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, 0, wlmtk_window_element(fw_ptr->window_ptr)->x);
     BS_TEST_VERIFY_EQ(test_ptr, 0, wlmtk_window_element(fw_ptr->window_ptr)->y);
 
     // Starts a move for the window. Will move it...
-    wlmtk_workspace_begin_window_move(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_begin_window_move(ws_ptr, fw_ptr->window_ptr);
     wlmtk_element_pointer_motion(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr), 1, 2, 43);
+        wlmtk_workspace_element(ws_ptr), 1, 2, 43);
     BS_TEST_VERIFY_EQ(test_ptr, 1, wlmtk_window_element(fw_ptr->window_ptr)->x);
     BS_TEST_VERIFY_EQ(test_ptr, 2, wlmtk_window_element(fw_ptr->window_ptr)->y);
 
@@ -1179,80 +1186,80 @@ void test_move(bs_test_t *test_ptr)
         .time_msec = 44,
     };
     wlmtk_element_pointer_button(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr),
+        wlmtk_workspace_element(ws_ptr),
         &button_event);
-    BS_TEST_VERIFY_EQ(test_ptr, NULL, fws_ptr->workspace_ptr->grabbed_window_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, NULL, ws_ptr->grabbed_window_ptr);
 
     // More motion, no longer updates the position.
     wlmtk_element_pointer_motion(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr), 3, 4, 45);
+        wlmtk_workspace_element(ws_ptr), 3, 4, 45);
     BS_TEST_VERIFY_EQ(test_ptr, 1, wlmtk_window_element(fw_ptr->window_ptr)->x);
     BS_TEST_VERIFY_EQ(test_ptr, 2, wlmtk_window_element(fw_ptr->window_ptr)->y);
 
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw_ptr->window_ptr);
 
     wlmtk_fake_window_destroy(fw_ptr);
-    wlmtk_fake_workspace_destroy(fws_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
 /** Tests moving a window that unmaps during the move. */
 void test_unmap_during_move(bs_test_t *test_ptr)
 {
-    wlmtk_fake_workspace_t *fws_ptr = wlmtk_fake_workspace_create(1024, 768);
-    BS_ASSERT(NULL != fws_ptr);
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
 
     wlmtk_element_pointer_motion(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr), 0, 0, 42);
+        wlmtk_workspace_element(ws_ptr), 0, 0, 42);
 
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw_ptr->window_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, 0, wlmtk_window_element(fw_ptr->window_ptr)->x);
     BS_TEST_VERIFY_EQ(test_ptr, 0, wlmtk_window_element(fw_ptr->window_ptr)->y);
 
     // Starts a move for the window. Will move it...
-    wlmtk_workspace_begin_window_move(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_begin_window_move(ws_ptr, fw_ptr->window_ptr);
     wlmtk_element_pointer_motion(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr), 1, 2, 43);
+        wlmtk_workspace_element(ws_ptr), 1, 2, 43);
     BS_TEST_VERIFY_EQ(test_ptr, 1, wlmtk_window_element(fw_ptr->window_ptr)->x);
     BS_TEST_VERIFY_EQ(test_ptr, 2, wlmtk_window_element(fw_ptr->window_ptr)->y);
 
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
-    BS_TEST_VERIFY_EQ(test_ptr, NULL, fws_ptr->workspace_ptr->grabbed_window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw_ptr->window_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, NULL, ws_ptr->grabbed_window_ptr);
 
     // More motion, no longer updates the position.
     wlmtk_element_pointer_motion(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr), 3, 4, 45);
+        wlmtk_workspace_element(ws_ptr), 3, 4, 45);
     BS_TEST_VERIFY_EQ(test_ptr, 1, wlmtk_window_element(fw_ptr->window_ptr)->x);
     BS_TEST_VERIFY_EQ(test_ptr, 2, wlmtk_window_element(fw_ptr->window_ptr)->y);
 
 
     // More motion, no longer updates the position.
     wlmtk_element_pointer_motion(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr), 3, 4, 45);
+        wlmtk_workspace_element(ws_ptr), 3, 4, 45);
     BS_TEST_VERIFY_EQ(test_ptr, 1, wlmtk_window_element(fw_ptr->window_ptr)->x);
     BS_TEST_VERIFY_EQ(test_ptr, 2, wlmtk_window_element(fw_ptr->window_ptr)->y);
 
     wlmtk_fake_window_destroy(fw_ptr);
-    wlmtk_fake_workspace_destroy(fws_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
 /** Tests resizing a window. */
 void test_resize(bs_test_t *test_ptr)
 {
-    wlmtk_fake_workspace_t *fws_ptr = wlmtk_fake_workspace_create(1024, 768);
-    BS_ASSERT(NULL != fws_ptr);
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
 
     wlmtk_fake_window_t *fw_ptr = wlmtk_fake_window_create();
-    BS_ASSERT(NULL != fw_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fw_ptr);
     wlmtk_window_request_position_and_size(fw_ptr->window_ptr, 0, 0, 40, 20);
     wlmtk_fake_window_commit_size(fw_ptr);
-    wlmtk_element_pointer_motion(
-        &fws_ptr->fake_parent_ptr->super_element, 0, 0, 42);
 
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_element_pointer_motion(wlmtk_workspace_element(ws_ptr),  0, 0, 42);
+
+    wlmtk_workspace_map_window(ws_ptr, fw_ptr->window_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, 0, wlmtk_window_element(fw_ptr->window_ptr)->x);
     BS_TEST_VERIFY_EQ(test_ptr, 0, wlmtk_window_element(fw_ptr->window_ptr)->y);
     int width, height;
@@ -1262,10 +1269,9 @@ void test_resize(bs_test_t *test_ptr)
 
     // Starts a resize for the window. Will resize & move it...
     wlmtk_workspace_begin_window_resize(
-        fws_ptr->workspace_ptr, fw_ptr->window_ptr, WLR_EDGE_TOP | WLR_EDGE_LEFT);
+        ws_ptr, fw_ptr->window_ptr, WLR_EDGE_TOP | WLR_EDGE_LEFT);
     fw_ptr->fake_content_ptr->serial = 1;  // The serial.
-    wlmtk_element_pointer_motion(
-        &fws_ptr->fake_parent_ptr->super_element, 1, 2, 44);
+    wlmtk_element_pointer_motion(wlmtk_workspace_element(ws_ptr), 1, 2, 44);
     BS_TEST_VERIFY_EQ(test_ptr, 0, wlmtk_window_element(fw_ptr->window_ptr)->x);
     BS_TEST_VERIFY_EQ(test_ptr, 0, wlmtk_window_element(fw_ptr->window_ptr)->y);
     BS_TEST_VERIFY_EQ(test_ptr, 39, fw_ptr->fake_content_ptr->requested_width);
@@ -1285,21 +1291,78 @@ void test_resize(bs_test_t *test_ptr)
         .time_msec = 44,
     };
     wlmtk_element_pointer_button(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr),
+        wlmtk_workspace_element(ws_ptr),
         &button_event);
-    BS_TEST_VERIFY_EQ(test_ptr, NULL, fws_ptr->workspace_ptr->grabbed_window_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, NULL, ws_ptr->grabbed_window_ptr);
 
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw_ptr->window_ptr);
     wlmtk_fake_window_destroy(fw_ptr);
-    wlmtk_fake_workspace_destroy(fws_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
+}
+
+/* ------------------------------------------------------------------------- */
+/** Tests enabling or disabling the workspace. */
+void test_enable(bs_test_t *test_ptr)
+{
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
+    wlmtk_workspace_enable(ws_ptr, false);
+
+    // Map while disabled: Not activated.
+    wlmtk_fake_window_t *fw1_ptr = wlmtk_fake_window_create();
+    wlmtk_workspace_map_window(ws_ptr, fw1_ptr->window_ptr);
+    BS_TEST_VERIFY_FALSE(
+        test_ptr,
+        wlmtk_window_is_activated(fw1_ptr->window_ptr));
+
+    // Enable: Activates the earlier-mapped window.
+    wlmtk_workspace_enable(ws_ptr, true);
+    BS_TEST_VERIFY_TRUE(
+        test_ptr,
+        wlmtk_window_is_activated(fw1_ptr->window_ptr));
+
+    // Disable: De-activate the window
+    wlmtk_workspace_enable(ws_ptr, false);
+    BS_TEST_VERIFY_FALSE(
+        test_ptr,
+        wlmtk_window_is_activated(fw1_ptr->window_ptr));
+
+    // Maps another window, while de-activated: Also don't activate.
+    wlmtk_fake_window_t *fw2_ptr = wlmtk_fake_window_create();
+    wlmtk_workspace_map_window(ws_ptr, fw2_ptr->window_ptr);
+    BS_TEST_VERIFY_FALSE(
+        test_ptr,
+        wlmtk_window_is_activated(fw2_ptr->window_ptr));
+
+    // Enable: Activates the window just mapped before.
+    wlmtk_workspace_enable(ws_ptr, true);
+    BS_TEST_VERIFY_TRUE(
+        test_ptr,
+        wlmtk_window_is_activated(fw2_ptr->window_ptr));
+
+    // Unmaps while de-activated. Enabling after should still activate fw1.
+    wlmtk_workspace_enable(ws_ptr, false);
+    wlmtk_workspace_unmap_window(ws_ptr, fw2_ptr->window_ptr);
+    wlmtk_fake_window_destroy(fw2_ptr);
+    BS_TEST_VERIFY_FALSE(
+        test_ptr,
+        wlmtk_window_is_activated(fw1_ptr->window_ptr));
+    wlmtk_workspace_enable(ws_ptr, true);
+    BS_TEST_VERIFY_TRUE(
+        test_ptr,
+        wlmtk_window_is_activated(fw1_ptr->window_ptr));
+
+    wlmtk_workspace_unmap_window(ws_ptr, fw1_ptr->window_ptr);
+    wlmtk_fake_window_destroy(fw1_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
 /** Tests window activation. */
 void test_activate(bs_test_t *test_ptr)
 {
-    wlmtk_fake_workspace_t *fws_ptr = wlmtk_fake_workspace_create(1024, 768);
-    BS_ASSERT(NULL != fws_ptr);
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
 
     // Window 1: from (0, 0) to (100, 100)
     wlmtk_fake_window_t *fw1_ptr = wlmtk_fake_window_create();
@@ -1311,7 +1374,7 @@ void test_activate(bs_test_t *test_ptr)
         wlmtk_window_is_activated(fw1_ptr->window_ptr));
 
     // Window 1 is mapped => it's activated.
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw1_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw1_ptr->window_ptr);
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_window_is_activated(fw1_ptr->window_ptr));
@@ -1325,7 +1388,7 @@ void test_activate(bs_test_t *test_ptr)
     BS_TEST_VERIFY_FALSE(
         test_ptr,
         wlmtk_window_is_activated(fw2_ptr->window_ptr));
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw2_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw2_ptr->window_ptr);
     BS_TEST_VERIFY_FALSE(
         test_ptr,
         wlmtk_window_is_activated(fw1_ptr->window_ptr));
@@ -1337,7 +1400,7 @@ void test_activate(bs_test_t *test_ptr)
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_element_pointer_motion(
-            &fws_ptr->fake_parent_ptr->super_element,  50, 50, 0));
+            wlmtk_workspace_element(ws_ptr), 50, 50, 0));
     BS_TEST_VERIFY_FALSE(
         test_ptr,
         wlmtk_window_is_activated(fw1_ptr->window_ptr));
@@ -1351,7 +1414,7 @@ void test_activate(bs_test_t *test_ptr)
         .type = WLMTK_BUTTON_DOWN,
     };
     wlmtk_element_pointer_button(
-        wlmtk_workspace_element(fws_ptr->workspace_ptr),
+        wlmtk_workspace_element(ws_ptr),
         &button_event);
     BS_TEST_VERIFY_TRUE(
         test_ptr,
@@ -1361,7 +1424,7 @@ void test_activate(bs_test_t *test_ptr)
         wlmtk_window_is_activated(fw2_ptr->window_ptr));
 
     // Unmap window 1. Now window 2 gets activated.
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw1_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw1_ptr->window_ptr);
     BS_TEST_VERIFY_FALSE(
         test_ptr,
         wlmtk_window_is_activated(fw1_ptr->window_ptr));
@@ -1370,28 +1433,28 @@ void test_activate(bs_test_t *test_ptr)
         wlmtk_window_is_activated(fw2_ptr->window_ptr));
 
     // Unmap the remaining window 2. Nothing is activated.
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw2_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw2_ptr->window_ptr);
     BS_TEST_VERIFY_FALSE(
         test_ptr,
         wlmtk_window_is_activated(fw2_ptr->window_ptr));
 
     wlmtk_fake_window_destroy(fw2_ptr);
     wlmtk_fake_window_destroy(fw1_ptr);
-    wlmtk_fake_workspace_destroy(fws_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
 /** Tests cycling through windows. */
 void test_activate_cycling(bs_test_t *test_ptr)
 {
-    wlmtk_fake_workspace_t *fws_ptr = wlmtk_fake_workspace_create(1024, 768);
-    BS_ASSERT(NULL != fws_ptr);
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_create_for_test(1024, 768, 0);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws_ptr);
     bs_dllist_t *windows_ptr = wlmtk_workspace_get_windows_dllist(
-        fws_ptr->workspace_ptr);
+        ws_ptr);
 
     // Window 1 gets mapped: Activated and on top.
     wlmtk_fake_window_t *fw1_ptr = wlmtk_fake_window_create();
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw1_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw1_ptr->window_ptr);
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_window_is_activated(fw1_ptr->window_ptr));
@@ -1402,7 +1465,7 @@ void test_activate_cycling(bs_test_t *test_ptr)
 
     // Window 2 gets mapped: Activated and on top.
     wlmtk_fake_window_t *fw2_ptr = wlmtk_fake_window_create();
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw2_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw2_ptr->window_ptr);
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_window_is_activated(fw2_ptr->window_ptr));
@@ -1413,7 +1476,7 @@ void test_activate_cycling(bs_test_t *test_ptr)
 
     // Window 3 gets mapped: Activated and on top.
     wlmtk_fake_window_t *fw3_ptr = wlmtk_fake_window_create();
-    wlmtk_workspace_map_window(fws_ptr->workspace_ptr, fw3_ptr->window_ptr);
+    wlmtk_workspace_map_window(ws_ptr, fw3_ptr->window_ptr);
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_window_is_activated(fw3_ptr->window_ptr));
@@ -1424,7 +1487,7 @@ void test_activate_cycling(bs_test_t *test_ptr)
 
     // From mapping sequence: We have 3 -> 2 -> 1. Cycling brings us to
     // window 2, but must not change the top window.
-    wlmtk_workspace_activate_next_window(fws_ptr->workspace_ptr);
+    wlmtk_workspace_activate_next_window(ws_ptr);
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_window_is_activated(fw2_ptr->window_ptr));
@@ -1434,7 +1497,7 @@ void test_activate_cycling(bs_test_t *test_ptr)
         windows_ptr->head_ptr);
 
     // One more cycle: 1.
-    wlmtk_workspace_activate_next_window(fws_ptr->workspace_ptr);
+    wlmtk_workspace_activate_next_window(ws_ptr);
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_window_is_activated(fw1_ptr->window_ptr));
@@ -1444,7 +1507,7 @@ void test_activate_cycling(bs_test_t *test_ptr)
         windows_ptr->head_ptr);
 
     // One more cycle: Back at 3.
-    wlmtk_workspace_activate_next_window(fws_ptr->workspace_ptr);
+    wlmtk_workspace_activate_next_window(ws_ptr);
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_window_is_activated(fw3_ptr->window_ptr));
@@ -1454,7 +1517,7 @@ void test_activate_cycling(bs_test_t *test_ptr)
         windows_ptr->head_ptr);
 
     // Cycle backward: Gets us to 1.
-    wlmtk_workspace_activate_previous_window(fws_ptr->workspace_ptr);
+    wlmtk_workspace_activate_previous_window(ws_ptr);
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_window_is_activated(fw1_ptr->window_ptr));
@@ -1464,20 +1527,20 @@ void test_activate_cycling(bs_test_t *test_ptr)
         windows_ptr->head_ptr);
 
     // Raise: Must come to top.
-    wlmtk_workspace_raise_window(fws_ptr->workspace_ptr,
+    wlmtk_workspace_raise_window(ws_ptr,
                                  fw1_ptr->window_ptr);
     BS_TEST_VERIFY_EQ(
         test_ptr,
         wlmtk_dlnode_from_window(fw1_ptr->window_ptr),
         windows_ptr->head_ptr);
 
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw3_ptr->window_ptr);
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw2_ptr->window_ptr);
-    wlmtk_workspace_unmap_window(fws_ptr->workspace_ptr, fw1_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw3_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw2_ptr->window_ptr);
+    wlmtk_workspace_unmap_window(ws_ptr, fw1_ptr->window_ptr);
     wlmtk_fake_window_destroy(fw3_ptr);
     wlmtk_fake_window_destroy(fw2_ptr);
     wlmtk_fake_window_destroy(fw1_ptr);
-    wlmtk_fake_workspace_destroy(fws_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
 }
 
 /* == End of workspace.c =================================================== */
